"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var React = require('react');
var ReactDOM = require('react-dom');
var chai = require('chai');
var enzyme_1 = require('enzyme');
var graphql_tag_1 = require('graphql-tag');
var apollo_client_1 = require('apollo-client');
var errors_1 = require('apollo-client/errors');
var chaiEnzyme = require('chai-enzyme');
chai.use(chaiEnzyme());
var expect = chai.expect;
var mockNetworkInterface_1 = require('../../../mocks/mockNetworkInterface');
var components_1 = require('../../../mocks/components');
var graphql_1 = require('../../../../src/graphql');
describe('queries', function () {
    it('binds a query to props', function () {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var ContainerWithData = graphql_1.default(query)(function (_a) {
            var data = _a.data;
            expect(data).to.exist;
            expect(data.ownProps).to.not.exist;
            expect(data.loading).to.be.true;
            return null;
        });
        var wrapper = enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ContainerWithData, null)));
        wrapper.unmount();
        var _a;
    });
    it('includes the variables in the props', function () {
        var query = (_a = ["query people ($first: Int) { allPeople(first: $first) { people { name } } }"], _a.raw = ["query people ($first: Int) { allPeople(first: $first) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var variables = { first: 1 };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var ContainerWithData = graphql_1.default(query)(function (_a) {
            var data = _a.data;
            expect(data).to.exist;
            expect(data.variables).to.deep.equal(variables);
            return null;
        });
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ContainerWithData, {first: 1})));
        var _a;
    });
    it('does not swallow children errors', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var bar;
        var ContainerWithData = graphql_1.default(query)(function () {
            bar();
            return null;
        });
        try {
            enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ContainerWithData, null)));
            done(new Error('component should have thrown'));
        }
        catch (e) {
            expect(e).to.match(/TypeError/);
            done();
        }
        var _a;
    });
    it('executes a query', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                expect(props.data.loading).to.be.false;
                expect(props.data.allPeople).to.deep.equal(data.allPeople);
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('correctly rebuilds props on remount', function (done) {
        var query = (_a = ["query pollingPeople { allPeople(first: 1) { people { name } } }"], _a.raw = ["query pollingPeople { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Darth Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data }, newData: function () { return ({
                data: {
                    allPeople: { people: [{ name: "Darth Skywalker - " + Math.random() }] },
                }
            }); } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var wrapper, app, count = 0;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                if (count === 1) {
                    wrapper.unmount();
                    wrapper = enzyme_1.mount(app);
                }
                if (count === 10) {
                    wrapper.unmount();
                    done();
                }
                count++;
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, { options: { pollInterval: 10 } })
            ], Container);
            return Container;
        }(React.Component));
        ;
        app = React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null));
        wrapper = enzyme_1.mount(app);
        var _a;
    });
    it('correctly sets loading state on remounted forcefetch', function (done) {
        var query = (_a = ["query pollingPeople { allPeople(first: 1) { people { name } } }"], _a.raw = ["query pollingPeople { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Darth Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data }, delay: 10, newData: function () { return ({
                data: {
                    allPeople: { people: [{ name: "Darth Skywalker - " + Math.random() }] },
                },
            }); } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var wrapper, app, count = 0;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillMount = function () {
                if (count === 1) {
                    expect(this.props.data.loading).to.be.true;
                    count++;
                }
            };
            Container.prototype.componentWillReceiveProps = function (props) {
                if (count === 0) {
                    wrapper.unmount();
                    setTimeout(function () {
                        wrapper = enzyme_1.mount(app);
                    }, 5);
                }
                if (count === 2) {
                    expect(props.data.loading).to.be.false;
                    expect(props.data.allPeople).to.exist;
                    done();
                }
                count++;
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, { options: { forceFetch: true } })
            ], Container);
            return Container;
        }(React.Component));
        ;
        app = React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null));
        wrapper = enzyme_1.mount(app);
        var _a;
    });
    it('correctly sets loading state on remounted component with changed variables', function (done) {
        var query = (_a = ["\n      query remount($first: Int) { allPeople(first: $first) { people { name } } }\n    "], _a.raw = ["\n      query remount($first: Int) { allPeople(first: $first) { people { name } } }\n    "], graphql_tag_1.default(_a));
        var data = { allPeople: null };
        var variables = { first: 1 };
        var variables2 = { first: 2 };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables }, result: { data: data }, delay: 10 }, { request: { query: query, variables: variables2 }, result: { data: data }, delay: 10 });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var wrapper, render, count = 0;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillMount = function () {
                if (count === 1) {
                    expect(this.props.data.loading).to.be.true;
                    count++;
                }
            };
            Container.prototype.componentWillReceiveProps = function (props) {
                if (count === 0) {
                    wrapper.unmount();
                    setTimeout(function () {
                        wrapper = enzyme_1.mount(render(2));
                    }, 5);
                }
                if (count === 2) {
                    expect(props.data.loading).to.be.false;
                    done();
                }
                count++;
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, { options: function (_a) {
                        var first = _a.first;
                        return ({ variables: { first: first } });
                    } })
            ], Container);
            return Container;
        }(React.Component));
        ;
        render = function (first) { return (React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, {first: first}))); };
        wrapper = enzyme_1.mount(render(1));
        var _a;
    });
    it('correctly sets loading state on remounted component with changed variables (alt)', function (done) {
        var query = (_a = ["\n      query remount($name: String) { allPeople(name: $name) { people { name } } }\n    "], _a.raw = ["\n      query remount($name: String) { allPeople(name: $name) { people { name } } }\n    "], graphql_tag_1.default(_a));
        var data = { allPeople: null };
        var variables = { name: 'does-not-exist' };
        var variables2 = { name: 'nothing-either' };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables }, result: { data: data }, delay: 10 }, { request: { query: query, variables: variables2 }, result: { data: data }, delay: 10 });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var count = 0;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.render = function () {
                var loading = this.props.data.loading;
                if (count === 0)
                    expect(loading).to.be.true;
                if (count === 1)
                    expect(loading).to.be.false;
                if (count === 2)
                    expect(loading).to.be.true;
                if (count === 3) {
                    expect(loading).to.be.false;
                    done();
                }
                count++;
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        var main = document.createElement('DIV');
        main.id = 'main';
        document.body.appendChild(main);
        var render = function (props) {
            ReactDOM.render((React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, __assign({}, props)))), document.getElementById('main'));
        };
        render(variables);
        setTimeout(function () { return render(variables2); }, 1000);
        var _a;
    });
    it('correctly sets loading state on component with changed variables and unchanged result', function (done) {
        var query = (_a = ["\n       query remount($first: Int) { allPeople(first: $first) { people { name } } }\n     "], _a.raw = ["\n       query remount($first: Int) { allPeople(first: $first) { people { name } } }\n     "], graphql_tag_1.default(_a));
        var data = { allPeople: null };
        var variables = { first: 1 };
        var variables2 = { first: 2 };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables }, result: { data: data }, delay: 10 }, { request: { query: query, variables: variables2 }, result: { data: data }, delay: 10 });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var count = 0;
        var connect = function (component) {
            return (function (_super) {
                __extends(Container, _super);
                function Container(props) {
                    _super.call(this, props);
                    this.state = {
                        first: 1,
                    };
                    this.setFirst = this.setFirst.bind(this);
                }
                Container.prototype.setFirst = function (first) {
                    this.setState({ first: first });
                };
                Container.prototype.render = function () {
                    return React.createElement(component, {
                        first: this.state.first,
                        setFirst: this.setFirst
                    });
                };
                return Container;
            }(React.Component));
        };
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                var _this = this;
                if (count === 0) {
                    setTimeout(function () {
                        _this.props.setFirst(2);
                    }, 5);
                }
                if (count === 1) {
                    expect(this.props.data.loading).to.be.true;
                }
                if (count === 2) {
                    expect(props.data.loading).to.be.false;
                    done();
                }
                count++;
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                connect,
                graphql_1.default(query, { options: function (_a) {
                        var first = _a.first;
                        return ({ variables: { first: first } });
                    } })
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('executes a query with two root fields', function (done) {
        var query = (_a = ["query people {\n      allPeople(first: 1) { people { name } }\n      otherPeople(first: 1) { people { name } }\n    }"], _a.raw = ["query people {\n      allPeople(first: 1) { people { name } }\n      otherPeople(first: 1) { people { name } }\n    }"], graphql_tag_1.default(_a));
        var data = {
            allPeople: { people: [{ name: 'Luke Skywalker' }] },
            otherPeople: { people: [{ name: 'Luke Skywalker' }] },
        };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                expect(props.data.loading).to.be.false;
                expect(props.data.allPeople).to.deep.equal(data.allPeople);
                expect(props.data.otherPeople).to.deep.equal(data.otherPeople);
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('can unmount without error', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var ContainerWithData = graphql_1.default(query)(function () { return null; });
        var wrapper = enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ContainerWithData, null)));
        try {
            wrapper.unmount();
            done();
        }
        catch (e) {
            done(e);
        }
        var _a;
    });
    it('passes any GraphQL errors in props', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, error: new Error('boo') });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var ErrorContainer = (function (_super) {
            __extends(ErrorContainer, _super);
            function ErrorContainer() {
                _super.apply(this, arguments);
            }
            ErrorContainer.prototype.componentWillReceiveProps = function (_a) {
                var data = _a.data;
                expect(data.error).to.exist;
                expect(data.error).instanceof(errors_1.ApolloError);
                done();
            };
            ErrorContainer.prototype.render = function () {
                return null;
            };
            ErrorContainer = __decorate([
                graphql_1.default(query)
            ], ErrorContainer);
            return ErrorContainer;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ErrorContainer, null)));
        var _a;
    });
    it('maps props as variables if they match', function (done) {
        var query = (_a = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], _a.raw = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var variables = { first: 1 };
        var networkInterface = mockNetworkInterface_1.default({
            request: { query: query, variables: variables },
            result: { data: data },
        });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                expect(props.data.loading).to.be.false;
                expect(props.data.allPeople).to.deep.equal(data.allPeople);
                expect(props.data.variables).to.deep.equal(this.props.data.variables);
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, {first: 1})));
        var _a;
    });
    it('allows falsy values in the mapped variables from props', function (done) {
        var query = (_a = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], _a.raw = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var variables = { first: null };
        var networkInterface = mockNetworkInterface_1.default({
            request: { query: query, variables: variables },
            result: { data: data },
        });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                expect(props.data.loading).to.be.false;
                expect(props.data.allPeople).to.deep.equal(data.allPeople);
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, {first: null})));
        var _a;
    });
    it('don\'t error on optional required props', function () {
        var query = (_a = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], _a.raw = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var variables = { first: 1 };
        var networkInterface = mockNetworkInterface_1.default({
            request: { query: query, variables: variables },
            result: { data: data },
        });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = graphql_1.default(query)(function () { return null; });
        var error = null;
        try {
            enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, {frst: 1})));
        }
        catch (e) {
            error = e;
        }
        expect(error).to.be.null;
        var _a;
    });
    it('errors if the passed props don\'t contain the needed variables', function () {
        var query = (_a = ["\n      query people($first: Int!) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], _a.raw = ["\n      query people($first: Int!) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var variables = { first: 1 };
        var networkInterface = mockNetworkInterface_1.default({
            request: { query: query, variables: variables },
            result: { data: data },
        });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = graphql_1.default(query)(function () { return null; });
        try {
            enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, {frst: 1})));
        }
        catch (e) {
            expect(e).to.match(/Invariant Violation: The operation 'people'/);
        }
        var _a;
    });
    it('rebuilds the queries on prop change when using `options`', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var firstRun = true;
        var isDone = false;
        function options(props) {
            if (!firstRun) {
                expect(props.listId).to.equal(2);
                if (!isDone)
                    done();
                isDone = true;
            }
            return {};
        }
        ;
        var Container = graphql_1.default(query, { options: options })(function (props) { return null; });
        var ChangingProps = (function (_super) {
            __extends(ChangingProps, _super);
            function ChangingProps() {
                _super.apply(this, arguments);
                this.state = { listId: 1 };
            }
            ChangingProps.prototype.componentDidMount = function () {
                var _this = this;
                setTimeout(function () {
                    firstRun = false;
                    _this.setState({ listId: 2 });
                }, 50);
            };
            ChangingProps.prototype.render = function () {
                return React.createElement(Container, {listId: this.state.listId});
            };
            return ChangingProps;
        }(React.Component));
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ChangingProps, null)));
        var _a;
    });
    it('allows you to skip a query', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var queryExecuted;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                queryExecuted = true;
            };
            Container.prototype.render = function () {
                expect(this.props.data.loading).to.be.false;
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, { options: function () { return ({ skip: true }); } })
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        setTimeout(function () {
            if (!queryExecuted) {
                done();
                return;
            }
            done(new Error('query ran even though skip present'));
        }, 25);
        var _a;
    });
    it('reruns the query if it changes', function (done) {
        var count = 0;
        var query = (_a = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], _a.raw = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], graphql_tag_1.default(_a));
        var data1 = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var variables1 = { first: 1 };
        var data2 = { allPeople: { people: [{ name: 'Leia Skywalker' }] } };
        var variables2 = { first: 2 };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables1 }, result: { data: data1 } }, { request: { query: query, variables: variables2 }, result: { data: data2 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (_a) {
                var data = _a.data;
                if (count === 1 && data.loading) {
                    expect(data.allPeople).to.deep.equal(data1.allPeople);
                }
                if (count === 1 && !data.loading && this.props.data.loading) {
                    expect(data.allPeople).to.deep.equal(data2.allPeople);
                    done();
                }
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, {
                    options: function (props) { return ({ variables: props, returnPartialData: count === 0 }); },
                })
            ], Container);
            return Container;
        }(React.Component));
        ;
        var ChangingProps = (function (_super) {
            __extends(ChangingProps, _super);
            function ChangingProps() {
                _super.apply(this, arguments);
                this.state = { first: 1 };
            }
            ChangingProps.prototype.componentDidMount = function () {
                var _this = this;
                setTimeout(function () {
                    count++;
                    _this.setState({ first: 2 });
                }, 50);
            };
            ChangingProps.prototype.render = function () {
                return React.createElement(Container, {first: this.state.first});
            };
            return ChangingProps;
        }(React.Component));
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ChangingProps, null)));
        var _a;
    });
    it('reruns the query if just the variables change', function (done) {
        var count = 0;
        var query = (_a = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], _a.raw = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], graphql_tag_1.default(_a));
        var data1 = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var variables1 = { first: 1 };
        var data2 = { allPeople: { people: [{ name: 'Leia Skywalker' }] } };
        var variables2 = { first: 2 };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables1 }, result: { data: data1 } }, { request: { query: query, variables: variables2 }, result: { data: data2 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (_a) {
                var data = _a.data;
                if (count === 1 && data.loading) {
                    expect(data.allPeople).to.deep.equal(data1.allPeople);
                }
                if (count === 1 && !data.loading && this.props.data.loading) {
                    expect(data.allPeople).to.deep.equal(data2.allPeople);
                    done();
                }
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, { options: function (props) { return ({ variables: props }); } })
            ], Container);
            return Container;
        }(React.Component));
        ;
        var ChangingProps = (function (_super) {
            __extends(ChangingProps, _super);
            function ChangingProps() {
                _super.apply(this, arguments);
                this.state = { first: 1 };
            }
            ChangingProps.prototype.componentDidMount = function () {
                var _this = this;
                setTimeout(function () {
                    count++;
                    _this.setState({ first: 2 });
                }, 50);
            };
            ChangingProps.prototype.render = function () {
                return React.createElement(Container, {first: this.state.first});
            };
            return ChangingProps;
        }(React.Component));
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ChangingProps, null)));
        var _a;
    });
    it('reruns the queries on prop change when using passed props', function (done) {
        var count = 0;
        var query = (_a = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], _a.raw = ["\n      query people($first: Int) {\n        allPeople(first: $first) { people { name } }\n      }\n    "], graphql_tag_1.default(_a));
        var data1 = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var variables1 = { first: 1 };
        var data2 = { allPeople: { people: [{ name: 'Leia Skywalker' }] } };
        var variables2 = { first: 2 };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables1 }, result: { data: data1 } }, { request: { query: query, variables: variables2 }, result: { data: data2 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (_a) {
                var data = _a.data;
                if (count === 1 && data.loading) {
                    expect(data.allPeople).to.deep.equal(data1.allPeople);
                }
                if (count === 1 && !data.loading && this.props.data.loading) {
                    expect(data.allPeople).to.deep.equal(data2.allPeople);
                    done();
                }
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        var ChangingProps = (function (_super) {
            __extends(ChangingProps, _super);
            function ChangingProps() {
                _super.apply(this, arguments);
                this.state = { first: 1 };
            }
            ChangingProps.prototype.componentDidMount = function () {
                var _this = this;
                setTimeout(function () {
                    count++;
                    _this.setState({ first: 2 });
                }, 50);
            };
            ChangingProps.prototype.render = function () {
                return React.createElement(Container, {first: this.state.first});
            };
            return ChangingProps;
        }(React.Component));
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ChangingProps, null)));
        var _a;
    });
    it('exposes refetch as part of the props api', function (done) {
        var query = (_a = ["query people($first: Int) { allPeople(first: $first) { people { name } } }"], _a.raw = ["query people($first: Int) { allPeople(first: $first) { people { name } } }"], graphql_tag_1.default(_a));
        var variables = { first: 1 };
        var data1 = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables }, result: { data: data1 } }, { request: { query: query, variables: variables }, result: { data: data1 } }, { request: { query: query, variables: { first: 2 } }, result: { data: data1 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var hasRefetched, count = 0;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillMount = function () {
                expect(this.props.data.refetch).to.be.exist;
                expect(this.props.data.refetch).to.be.instanceof(Function);
            };
            Container.prototype.componentWillReceiveProps = function (_a) {
                var data = _a.data;
                if (count === 0)
                    expect(data.loading).to.be.false;
                if (count === 1)
                    expect(data.loading).to.be.true;
                if (count === 2)
                    expect(data.loading).to.be.false;
                if (count === 3)
                    expect(data.loading).to.be.true;
                if (count === 4)
                    expect(data.loading).to.be.false;
                count++;
                if (hasRefetched)
                    return;
                hasRefetched = true;
                expect(data.refetch).to.be.exist;
                expect(data.refetch).to.be.instanceof(Function);
                data.refetch()
                    .then(function (result) {
                    expect(result.data).to.deep.equal(data1);
                    data.refetch({ first: 2 })
                        .then(function (response) {
                        expect(response.data).to.deep.equal(data1);
                        expect(data.allPeople).to.deep.equal(data1.allPeople);
                        done();
                    });
                })
                    .catch(done);
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, {first: 1})));
        var _a;
    });
    it('exposes fetchMore as part of the props api', function (done) {
        var query = (_a = ["\n      query people($skip: Int, $first: Int) { allPeople(first: $first, skip: $skip) { people { name } } }\n    "], _a.raw = ["\n      query people($skip: Int, $first: Int) { allPeople(first: $first, skip: $skip) { people { name } } }\n    "], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var data1 = { allPeople: { people: [{ name: 'Leia Skywalker' }] } };
        var variables = { skip: 1, first: 1 };
        var variables2 = { skip: 2, first: 1 };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: variables }, result: { data: data } }, { request: { query: query, variables: variables2 }, result: { data: data1 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var count = 0;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillMount = function () {
                expect(this.props.data.fetchMore).to.be.exist;
                expect(this.props.data.fetchMore).to.be.instanceof(Function);
            };
            Container.prototype.componentWillReceiveProps = function (props) {
                if (count === 0) {
                    expect(props.data.fetchMore).to.be.exist;
                    expect(props.data.fetchMore).to.be.instanceof(Function);
                    props.data.fetchMore({
                        variables: { skip: 2 },
                        updateQuery: function (prev, _a) {
                            var fetchMoreResult = _a.fetchMoreResult;
                            return ({
                                allPeople: {
                                    people: prev.allPeople.people.concat(fetchMoreResult.data.allPeople.people),
                                },
                            });
                        },
                    });
                }
                else if (count === 1) {
                    expect(props.data.variables).to.deep.equal(variables2);
                    expect(props.data.loading).to.be.true;
                    expect(props.data.allPeople).to.deep.equal(data.allPeople);
                }
                else if (count === 2) {
                    expect(props.data.variables).to.deep.equal(variables2);
                    expect(props.data.loading).to.be.false;
                    expect(props.data.allPeople.people).to.deep.equal(data.allPeople.people.concat(data1.allPeople.people));
                    done();
                }
                else {
                    done(new Error('should not reach this point'));
                }
                count++;
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, { options: function () { return ({ variables: variables }); } })
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('exposes stopPolling as part of the props api', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (_a) {
                var data = _a.data;
                expect(data.stopPolling).to.be.exist;
                expect(data.stopPolling).to.be.instanceof(Function);
                expect(data.stopPolling).to.not.throw;
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('exposes startPolling as part of the props api', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (_a) {
                var data = _a.data;
                expect(data.startPolling).to.be.exist;
                expect(data.startPolling).to.be.instanceof(Function);
                expect(data.startPolling).to.not.throw;
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('resets the loading state after a refetched query', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var data2 = { allPeople: { people: [{ name: 'Leia Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } }, { request: { query: query }, result: { data: data2 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var isRefectching;
        var refetched;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                if (refetched) {
                    expect(props.data.loading).to.be.false;
                    expect(props.data.allPeople).to.deep.equal(data2.allPeople);
                    done();
                    return;
                }
                if (isRefectching) {
                    isRefectching = false;
                    refetched = true;
                    expect(props.data.loading).to.be.true;
                    expect(props.data.allPeople).to.deep.equal(data.allPeople);
                    return;
                }
                if (!isRefectching) {
                    isRefectching = true;
                    props.data.refetch();
                }
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('resets the loading state after a refetched query even if the data doesn\'t change', function (d) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } }, { request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var isRefectching;
        var refetched;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                if (refetched) {
                    expect(props.data.loading).to.be.false;
                    expect(props.data.allPeople).to.deep.equal(data.allPeople);
                    d();
                    return;
                }
                if (isRefectching) {
                    isRefectching = false;
                    refetched = true;
                    expect(props.data.loading).to.be.true;
                    expect(props.data.allPeople).to.deep.equal(data.allPeople);
                    return;
                }
                if (!isRefectching) {
                    isRefectching = true;
                    props.data.refetch();
                }
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('allows a polling query to be created', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var data2 = { allPeople: { people: [{ name: 'Leia Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } }, { request: { query: query }, result: { data: data2 } }, { request: { query: query }, result: { data: data2 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var count = 0;
        var Container = graphql_1.default(query, { options: function () { return ({ pollInterval: 75 }); } })(function () {
            count++;
            return null;
        });
        var wrapper = enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        setTimeout(function () {
            expect(count).to.equal(3);
            wrapper.unmount();
            done();
        }, 160);
        var _a;
    });
    it('allows custom mapping of a result to props', function () {
        var query = (_a = ["query thing { getThing { thing } }"], _a.raw = ["query thing { getThing { thing } }"], graphql_tag_1.default(_a));
        var data = { getThing: { thing: true } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var props = function (_a) {
            var data = _a.data;
            return ({ showSpinner: data.loading });
        };
        var ContainerWithData = graphql_1.default(query, { props: props })(function (_a) {
            var showSpinner = _a.showSpinner;
            expect(showSpinner).to.be.true;
            return null;
        });
        var wrapper = enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ContainerWithData, null)));
        wrapper.unmount();
        var _a;
    });
    it('allows custom mapping of a result to props that includes the passed props', function () {
        var query = (_a = ["query thing { getThing { thing } }"], _a.raw = ["query thing { getThing { thing } }"], graphql_tag_1.default(_a));
        var data = { getThing: { thing: true } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var props = function (_a) {
            var data = _a.data, ownProps = _a.ownProps;
            expect(ownProps.sample).to.equal(1);
            return { showSpinner: data.loading };
        };
        var ContainerWithData = graphql_1.default(query, { props: props })(function (_a) {
            var showSpinner = _a.showSpinner;
            expect(showSpinner).to.be.true;
            return null;
        });
        var wrapper = enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ContainerWithData, {sample: 1})));
        wrapper.unmount();
        var _a;
    });
    it('allows custom mapping of a result to props', function (done) {
        var query = (_a = ["query thing { getThing { thing } }"], _a.raw = ["query thing { getThing { thing } }"], graphql_tag_1.default(_a));
        var data = { getThing: { thing: true } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                expect(props.thingy).to.deep.equal(data.getThing);
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, { props: function (_a) {
                        var data = _a.data;
                        return ({ thingy: data.getThing });
                    } })
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('allows context through updates', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                expect(props.data.loading).to.be.false;
                expect(props.data.allPeople).to.deep.equal(data.allPeople);
            };
            Container.prototype.render = function () {
                return React.createElement("div", null, this.props.children);
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        var ContextContainer = (function (_super) {
            __extends(ContextContainer, _super);
            function ContextContainer(props) {
                _super.call(this, props);
                this.state = { color: 'purple' };
            }
            ContextContainer.prototype.getChildContext = function () {
                return { color: this.state.color };
            };
            ContextContainer.prototype.componentDidMount = function () {
                var _this = this;
                setTimeout(function () {
                    _this.setState({ color: 'green' });
                }, 50);
            };
            ContextContainer.prototype.render = function () {
                return React.createElement("div", null, this.props.children);
            };
            return ContextContainer;
        }(React.Component));
        ContextContainer.childContextTypes = {
            color: React.PropTypes.string,
        };
        var count = 0;
        var ChildContextContainer = (function (_super) {
            __extends(ChildContextContainer, _super);
            function ChildContextContainer() {
                _super.apply(this, arguments);
            }
            ChildContextContainer.prototype.render = function () {
                var color = this.context.color;
                if (count === 0)
                    expect(color).to.eq('purple');
                if (count === 1) {
                    expect(color).to.eq('green');
                    done();
                }
                count++;
                return React.createElement("div", null, this.props.children);
            };
            return ChildContextContainer;
        }(React.Component));
        ChildContextContainer.contextTypes = {
            color: React.PropTypes.string,
        };
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(ContextContainer, null, React.createElement(Container, null, React.createElement(ChildContextContainer, null)))));
        var _a;
    });
    it('exposes updateQuery as part of the props api', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (_a) {
                var data = _a.data;
                expect(data.updateQuery).to.be.exist;
                expect(data.updateQuery).to.be.instanceof(Function);
                expect(data.updateQuery).to.not.throw;
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('exposes updateQuery as part of the props api during componentWillMount', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillMount = function () {
                expect(this.props.updateQuery).to.be.exist;
                expect(this.props.updateQuery).to.be.instanceof(Function);
                expect(this.props.updateQuery).to.not.throw;
                done();
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('allows updating query results after query has finished', function (done) {
        var query = (_a = ["query people { allPeople(first: 1) { people { name } } }"], _a.raw = ["query people { allPeople(first: 1) { people { name } } }"], graphql_tag_1.default(_a));
        var data = { allPeople: { people: [{ name: 'Luke Skywalker' }] } };
        var data2 = { allPeople: { people: [{ name: 'Leia Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query }, result: { data: data } }, { request: { query: query }, result: { data: data2 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var isUpdated;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                if (isUpdated) {
                    expect(props.data.allPeople).to.deep.equal(data2.allPeople);
                    done();
                    return;
                }
                else {
                    isUpdated = true;
                    props.data.updateQuery(function (prev) {
                        return data2;
                    });
                }
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query)
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
    it('reruns props function after query results change via fetchMore', function (done) {
        var query = (_a = ["query people($cursor: Int) {\n      allPeople(cursor: $cursor) { cursor, people { name } }\n    }"], _a.raw = ["query people($cursor: Int) {\n      allPeople(cursor: $cursor) { cursor, people { name } }\n    }"], graphql_tag_1.default(_a));
        var vars1 = { cursor: null };
        var data1 = { allPeople: { cursor: 1, people: [{ name: 'Luke Skywalker' }] } };
        var vars2 = { cursor: 1 };
        var data2 = { allPeople: { cursor: 2, people: [{ name: 'Leia Skywalker' }] } };
        var networkInterface = mockNetworkInterface_1.default({ request: { query: query, variables: vars1 }, result: { data: data1 } }, { request: { query: query, variables: vars2 }, result: { data: data2 } });
        var client = new apollo_client_1.default({ networkInterface: networkInterface });
        var isUpdated = false;
        var Container = (function (_super) {
            __extends(Container, _super);
            function Container() {
                _super.apply(this, arguments);
            }
            Container.prototype.componentWillReceiveProps = function (props) {
                if (props.loading) {
                    return;
                }
                else if (isUpdated) {
                    expect(props.people.length).to.equal(2);
                    done();
                    return;
                }
                else {
                    isUpdated = true;
                    expect(props.people).to.deep.equal(data1.allPeople.people);
                    props.getMorePeople();
                }
            };
            Container.prototype.render = function () {
                return null;
            };
            Container = __decorate([
                graphql_1.default(query, {
                    options: { variables: { cursor: null } },
                    props: function (_a) {
                        var _b = _a.data, loading = _b.loading, allPeople = _b.allPeople, fetchMore = _b.fetchMore;
                        if (loading)
                            return { loading: loading };
                        var cursor = allPeople.cursor, people = allPeople.people;
                        return {
                            people: people,
                            getMorePeople: function () { return fetchMore({
                                variables: { cursor: cursor },
                                updateQuery: function (prev, _a) {
                                    var fetchMoreResult = _a.fetchMoreResult;
                                    var _b = fetchMoreResult.data.allPeople, cursor = _b.cursor, people = _b.people;
                                    return {
                                        allPeople: {
                                            cursor: cursor,
                                            people: people.concat(prev.allPeople.people),
                                        },
                                    };
                                }
                            }); },
                        };
                    }
                })
            ], Container);
            return Container;
        }(React.Component));
        ;
        enzyme_1.mount(React.createElement(components_1.ProviderMock, {client: client}, React.createElement(Container, null)));
        var _a;
    });
});
//# sourceMappingURL=queries.js.map